@@ -74,25 +74,25 @@ test_applet_handle_size_change (PanelApplet *applet,
 				GtkLabel    *label)
 {
 	switch (size) {
-	case GNOME_PANEL_XX_SMALL:
+	case GNOME_Vertigo_PANEL_XX_SMALL:
 		gtk_label_set_markup (label, "<span size=\"xx-small\">Hello</span>");
 		break;
-	case GNOME_PANEL_X_SMALL:
+	case GNOME_Vertigo_PANEL_X_SMALL:
 		gtk_label_set_markup (label, "<span size=\"x-small\">Hello</span>");
 		break;
-	case GNOME_PANEL_SMALL:
+	case GNOME_Vertigo_PANEL_SMALL:
 		gtk_label_set_markup (label, "<span size=\"small\">Hello</span>");
 		break;
-	case GNOME_PANEL_MEDIUM:
+	case GNOME_Vertigo_PANEL_MEDIUM:
 		gtk_label_set_markup (label, "<span size=\"medium\">Hello</span>");
 		break;
-	case GNOME_PANEL_LARGE:
+	case GNOME_Vertigo_PANEL_LARGE:
 		gtk_label_set_markup (label, "<span size=\"large\">Hello</span>");
 		break;
-	case GNOME_PANEL_X_LARGE:
+	case GNOME_Vertigo_PANEL_X_LARGE:
 		gtk_label_set_markup (label, "<span size=\"x-large\">Hello</span>");
 		break;
-	case GNOME_PANEL_XX_LARGE:
+	case GNOME_Vertigo_PANEL_XX_LARGE:
 		gtk_label_set_markup (label, "<span size=\"xx-large\">Hello</span>");
 		break;
 	default:
@@ -105,19 +105,22 @@ static void
 test_applet_handle_background_change (PanelApplet               *applet,
 				      PanelAppletBackgroundType  type,
 				      GdkColor                  *color,
-				      const gchar               *pixmap,
+				      GdkPixmap                 *pixmap,
 				      GtkLabel                  *label)
 {
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
 		g_message ("Setting background to default");
+		gdk_window_set_back_pixmap (GTK_WIDGET (label)->window, NULL, FALSE);
 		break;
 	case PANEL_COLOR_BACKGROUND:
 		g_message ("Setting background to #%2x%2x%2x",
 			    color->red, color->green, color->blue);
+		gdk_window_set_back_pixmap (GTK_WIDGET (label)->window, NULL, FALSE);
 		break;
-	case PANEL_PIXMAP_BACKGOUND:
+	case PANEL_PIXMAP_BACKGROUND:
 		g_message ("Setting background to '%s'", pixmap);
+		gdk_window_set_back_pixmap (GTK_WIDGET (label)->window, pixmap, FALSE);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -125,27 +128,19 @@ test_applet_handle_background_change (PanelApplet               *applet,
 	}
 }
 
-static void
-test_applet_handle_save_yourself (PanelApplet               *applet,
-				  GtkLabel                  *label)
+static gboolean
+test_applet_fill (PanelApplet *applet)
 {
-	g_message ("saveYourself received");
-}
-
-static BonoboObject *
-test_applet_new (void)
-{
-	GtkWidget *applet;
 	GtkWidget *label;
 
 	label = gtk_label_new (NULL);
 
-	applet = panel_applet_new (label);
+	gtk_container_add (GTK_CONTAINER (applet), label);
 
-	gtk_widget_show_all (applet);
+	gtk_widget_show_all (GTK_WIDGET (applet));
 
 	test_applet_handle_size_change (PANEL_APPLET (applet),
-					GNOME_PANEL_MEDIUM,
+					GNOME_Vertigo_PANEL_MEDIUM,
 					GTK_LABEL (label));
 
 	panel_applet_setup_menu (PANEL_APPLET (applet),
@@ -170,25 +165,20 @@ test_applet_new (void)
 			  G_CALLBACK (test_applet_handle_background_change),
 			  label);
 
-	g_signal_connect (G_OBJECT (applet),
-			  "save_yourself",
-			  G_CALLBACK (test_applet_handle_save_yourself),
-			  label);
-			  
-	return BONOBO_OBJECT (panel_applet_get_control (PANEL_APPLET (applet)));
+	return TRUE;
 }
 
-static BonoboObject *
-test_applet_factory (BonoboGenericFactory *this,
-		     const gchar          *iid,
-		     gpointer              data)
+static gboolean
+test_applet_factory (PanelApplet *applet,
+		     const gchar *iid,
+		     gpointer     data)
 {
-	BonoboObject *applet = NULL;
+	gboolean retval = FALSE;
     
 	if (!strcmp (iid, "OAFIID:GNOME_Panel_TestBonoboApplet"))
-		applet = test_applet_new (); 
+		retval = test_applet_fill (applet); 
     
-	return applet;
+	return retval;
 }
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_Panel_TestBonoboApplet_Factory",
