@@ -10,7 +10,7 @@
 #include "drawer-widget.h"
 #include "border-widget.h"
 #include "floating-widget.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
 
 #include "multiscreen-stuff.h"
 
@@ -221,8 +221,8 @@ drawer_widget_close_drawer (DrawerWidget *drawer, GtkWidget *parentp)
 static void
 drawer_pos_hidebutton_click (BasePWidget *basep)
 {
-	Drawer *drawer = gtk_object_get_data (GTK_OBJECT (basep),
-					      DRAWER_PANEL_KEY);
+	Drawer *drawer = g_object_get_data (G_OBJECT (basep),
+					    DRAWER_PANEL_KEY);
 	PanelWidget *panel = PANEL_WIDGET (drawer->button->parent);
 	GtkWidget *parent = panel->panel_parent;
 
@@ -268,7 +268,7 @@ drawer_pos_get_pos(BasePWidget *basep,
 	    GTK_WIDGET_REALIZED (panel->master_widget) &&
 	    /*"allocated" data will be set on each allocation, until then,
 	      don't show the actual panel*/
-	    gtk_object_get_data(GTK_OBJECT(panel->master_widget),"allocated")) {
+	    g_object_get_data (G_OBJECT (panel->master_widget), "allocated")) {
 		int bx, by, bw, bh;
 		int px, py, pw, ph;
 		GtkWidget *ppanel; /*parent panel*/
@@ -367,7 +367,7 @@ drawer_widget_change_params (DrawerWidget *drawer,
 			     PanelBackType back_type,
 			     char *back_pixmap,
 			     gboolean fit_pixmap_bg,
-			     gboolean strech_pixmap_bg,
+			     gboolean stretch_pixmap_bg,
 			     gboolean rotate_pixmap_bg,
 			     GdkColor *back_color)
 {
@@ -421,7 +421,7 @@ drawer_widget_change_params (DrawerWidget *drawer,
 				    back_type,
 				    back_pixmap,
 				    fit_pixmap_bg,
-				    strech_pixmap_bg,
+				    stretch_pixmap_bg,
 				    rotate_pixmap_bg,
 				    back_color);
 				    
@@ -445,14 +445,15 @@ drawer_widget_change_orient (DrawerWidget *drawer,
 					     panel->back_type,
 					     panel->back_pixmap,
 					     panel->fit_pixmap_bg,
-					     panel->strech_pixmap_bg,
+					     panel->stretch_pixmap_bg,
 					     panel->rotate_pixmap_bg,
 					     &panel->back_color);
 	}
 }
 
 GtkWidget *
-drawer_widget_new (PanelOrient orient,
+drawer_widget_new (gchar *panel_id,
+		   PanelOrient orient,
 		   BasePMode mode,
 		   BasePState state,
 		   int sz,
@@ -461,7 +462,7 @@ drawer_widget_new (PanelOrient orient,
 		   PanelBackType back_type,
 		   char *back_pixmap,
 		   gboolean fit_pixmap_bg,
-		   gboolean strech_pixmap_bg,
+		   gboolean stretch_pixmap_bg,
 		   gboolean rotate_pixmap_bg,
 		   GdkColor *back_color)
 {
@@ -469,8 +470,8 @@ drawer_widget_new (PanelOrient orient,
 	DrawerPos *pos;
 	GtkOrientation porient;
 
-	drawer = gtk_type_new (DRAWER_TYPE_WIDGET);
-	drawer->pos = gtk_type_new (DRAWER_TYPE_POS);
+	drawer = g_object_new (DRAWER_TYPE_WIDGET, NULL);
+	drawer->pos = g_object_new (DRAWER_TYPE_POS, NULL);
 	pos = DRAWER_POS (drawer->pos);
 	pos->orient = orient;
 
@@ -484,7 +485,8 @@ drawer_widget_new (PanelOrient orient,
 		break;
 	}
 
-	basep_widget_construct (BASEP_WIDGET (drawer),
+	basep_widget_construct (panel_id, 
+				BASEP_WIDGET (drawer),
 				TRUE, TRUE,
 				0 /*FIXME */,
 				porient,
@@ -494,19 +496,9 @@ drawer_widget_new (PanelOrient orient,
 				back_type,
 				back_pixmap,
 				fit_pixmap_bg,
-				strech_pixmap_bg,
+				stretch_pixmap_bg,
 				rotate_pixmap_bg,
 				back_color);
 
 	return GTK_WIDGET (drawer);
 }
-
-#if 0
-void
-drawer_widget_restore_state(DrawerWidget *drawer)
-{
-	DRAWER_POS (BASEP_WIDGET (drawer)->pos)->temp_hidden = FALSE;
-	gtk_widget_queue_resize(GTK_WIDGET(drawer));
-	gtk_widget_show(GTK_WIDGET(drawer));
-}
-#endif
