@@ -23,7 +23,8 @@
 #include "applet.h"
 #include "button-widget.h"
 #include "drawer-widget.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
+#include "panel-gconf.h"
 #include "panel-util.h"
 #include "session.h"
 #include "xstuff.h"
@@ -34,53 +35,53 @@ extern gboolean commie_mode;
 extern int panels_to_sync;
 
 extern GtkTooltips *panel_tooltips;
+extern GSList *panels;
 
 #undef DRAWER_DEBUG
 
 static void
 properties_apply_callback(gpointer data)
 {
-#ifdef FIXME
 	Drawer       *drawer = data;
 
-	GtkWidget    *pixentry = gtk_object_get_data(GTK_OBJECT(drawer->properties),
-						     "pixmap");
-	GtkWidget    *tipentry = gtk_object_get_data(GTK_OBJECT(drawer->properties),
-						     "tooltip");
+	GtkWidget    *pixentry = g_object_get_data (G_OBJECT (drawer->properties),
+						    "pixmap");
+	GtkWidget    *tipentry = g_object_get_data (G_OBJECT (drawer->properties),
+						    "tooltip");
 	char         *s;
+	const char   *cs;
 
 	g_free (drawer->pixmap);
 	drawer->pixmap = NULL;
 	g_free (drawer->tooltip);
 	drawer->tooltip = NULL;
-	s = hack_icon_entry_get_icon (GNOME_ICON_ENTRY (pixentry));
+	s = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (pixentry));
 	if (string_empty (s)) {
 		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 							    "panel-drawer.png", TRUE, NULL);
-		button_widget_set_pixmap (BUTTON_WIDGET(drawer->button),
+		button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
 					  drawer->pixmap,-1);
 	} else {
-		if(button_widget_set_pixmap(BUTTON_WIDGET(drawer->button), s, -1))
-			drawer->pixmap = g_strdup(s);
+		if(button_widget_set_pixmap (BUTTON_WIDGET (drawer->button), s, -1))
+			drawer->pixmap = g_strdup (s);
 		else {
 			drawer->pixmap = gnome_program_locate_file (NULL, 
 								    GNOME_FILE_DOMAIN_PIXMAP, 
 								    "panel-drawer.png",
 								     TRUE, NULL);
-			button_widget_set_pixmap(BUTTON_WIDGET(drawer->button),
-						 drawer->pixmap, -1);
+			button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
+						  drawer->pixmap, -1);
 		}
 	}
 	g_free(s);
-	s = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
-	if (string_empty (s))
+	cs = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
+	if (string_empty (cs))
 		drawer->tooltip = NULL;
 	else
-		drawer->tooltip = g_strdup(s);
+		drawer->tooltip = g_strdup (cs);
 
 	gtk_tooltips_set_tip (panel_tooltips, drawer->button,
 			      drawer->tooltip, NULL);
-#endif
 }
 
 static void
@@ -93,7 +94,7 @@ properties_close_callback(GtkWidget *widget, gpointer data)
 static void
 set_toggle (GtkWidget *widget, gpointer data)
 {
-	PerPanelConfig *ppc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	PerPanelConfig *ppc = g_object_get_data (G_OBJECT (widget), "PerPanelConfig");
 	int *the_toggle = data;
 
 	*the_toggle = GTK_TOGGLE_BUTTON(widget)->active;
@@ -142,13 +143,13 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 			      drawer->tooltip,
 			      (UpdateFunction)panel_config_register_changes,
 			      ppc);
-	gtk_object_set_data(GTK_OBJECT(dialog),"tooltip",w);
+	g_object_set_data (G_OBJECT (dialog), "tooltip", w);
 	
 	w = create_icon_entry(table, "icon", 0, 2, _("Icon"),
 			      NULL, drawer->pixmap,
 			      (UpdateFunction)panel_config_register_changes,
 			      ppc);
-	gtk_object_set_data(GTK_OBJECT(dialog), "pixmap", w);
+	g_object_set_data (G_OBJECT (dialog), "pixmap", w);
 
 	f = gtk_frame_new(_("Applet appearance"));
 	gtk_container_add(GTK_CONTAINER(f),table);
@@ -161,7 +162,7 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 	/*we store this in w for later use!, so don't use w as temp from now
 	  on*/
 	w = button = gtk_check_button_new_with_label (_("Enable hidebutton"));
-	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	if (ppc->hidebuttons)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	g_signal_connect (G_OBJECT (button), "toggled", 
@@ -175,7 +176,7 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 			    button);
 	if (!ppc->hidebuttons)
 		gtk_widget_set_sensitive(button,FALSE);
-	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	if (ppc->hidebutton_pixmaps)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	g_signal_connect (G_OBJECT (button), "toggled", 
@@ -348,8 +349,8 @@ create_drawer_applet(GtkWidget * drawer_panel,
 	GTK_WIDGET_SET_FLAGS (drawer->button, GTK_NO_WINDOW);
 
 	g_signal_connect (G_OBJECT (drawer->button), "drag_data_get",
-			    G_CALLBACK (drag_data_get_cb),
-			    NULL);
+			  G_CALLBACK (drag_data_get_cb),
+			  NULL);
 
 	gtk_widget_show(drawer->button);
 
@@ -364,8 +365,7 @@ create_drawer_applet(GtkWidget * drawer_panel,
 	g_signal_connect (G_OBJECT (drawer->button), "leave_notify_event",
 			    G_CALLBACK (leave_notify_drawer), drawer);
 
-	gtk_object_set_user_data (GTK_OBJECT (drawer->button), drawer);
-	gtk_object_set_data (GTK_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
+	g_object_set_data (G_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
 	gtk_widget_queue_resize (GTK_WIDGET (drawer_panel));
 
 	return drawer;
@@ -375,7 +375,7 @@ static Drawer *
 create_empty_drawer_applet(const char *tooltip, const char *pixmap,
 			   PanelOrient orient)
 {
-	GtkWidget *dw = drawer_widget_new (orient,
+	GtkWidget *dw = drawer_widget_new (NULL, orient,
 					   BASEP_EXPLICIT_HIDE,
 					   BASEP_SHOWN,
 					   PANEL_SIZE_MEDIUM,
@@ -408,8 +408,8 @@ drawer_setup(Drawer *drawer)
 	if(BASEP_WIDGET(drawer->drawer)->state != BASEP_SHOWN) {
 		GtkRequisition chreq;
 		gtk_widget_size_request(drawer->drawer, &chreq);
-		gtk_widget_set_uposition(drawer->drawer,
-					 -chreq.width - 1, -chreq.height - 1);
+		gtk_window_move (GTK_WINDOW (drawer->drawer),
+				 -chreq.width - 1, -chreq.height - 1);
 	}
 	gtk_widget_show(drawer->drawer);
 }
@@ -420,20 +420,25 @@ button_size_alloc(GtkWidget *widget, GtkAllocation *alloc, Drawer *drawer)
 	if(!GTK_WIDGET_REALIZED(widget))
 		return;
 
-	gtk_widget_queue_resize(drawer->drawer);
+	gtk_widget_queue_resize (drawer->drawer);
 
-	gtk_object_set_data(GTK_OBJECT(widget),"allocated",GINT_TO_POINTER(1));
+	g_object_set_data (G_OBJECT (widget), "allocated", GINT_TO_POINTER (TRUE));
 }
 
 Drawer *
-load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
-		    PanelWidget *panel, int pos, gboolean exactpos)
+load_drawer_applet (gchar       *mypanel_id,
+		    const char  *pixmap,
+		    const char  *tooltip,
+		    PanelWidget *panel,
+		    int          pos,
+		    gboolean     exactpos,
+		    const char  *gconf_key)
 {
-	Drawer          *drawer;
+	Drawer      *drawer;
 	PanelOrient  orient = get_applet_orient (panel);
-	AppletInfo      *info;
+	AppletInfo  *info;
 
-	if (mypanel_id < 0) {
+	if (!mypanel_id) {
 		drawer = create_empty_drawer_applet (tooltip, pixmap, orient);
 		if (drawer != NULL)
 			panel_setup (drawer->drawer);
@@ -474,7 +479,8 @@ load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
 					      panel,
 					      pos,
 					      exactpos,
-					      APPLET_DRAWER);
+					      APPLET_DRAWER,
+					      gconf_key);
 
 		if (!info) {
 			gtk_widget_destroy (dw);
@@ -512,3 +518,95 @@ load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
 	return drawer;
 }
 
+void
+drawer_save_to_gconf (Drawer     *drawer,
+		      const char *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+
+	g_return_if_fail (drawer && BASEP_IS_WIDGET (drawer->drawer));
+	g_return_if_fail (PANEL_IS_WIDGET (BASEP_WIDGET (drawer->drawer)->panel));
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "parameters");
+	gconf_client_set_int (client, temp_key,
+			      g_slist_index (panels, BASEP_WIDGET (drawer->drawer)->panel),
+			      NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "unique-drawer-panel-id");
+	gconf_client_set_string (client, temp_key,
+				 PANEL_WIDGET (BASEP_WIDGET (drawer->drawer)->panel)->unique_id,
+				 NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "pixmap");
+	gconf_client_set_string (client, temp_key, drawer->pixmap, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "tooltip");
+	gconf_client_set_string (client, temp_key, drawer->tooltip, NULL);
+	g_free (temp_key);
+
+	g_free (profile);
+}
+
+void
+drawer_load_from_gconf (PanelWidget *panel_widget,
+			gint         position,
+			const char  *gconf_key,
+			gboolean     use_default)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+	int          panel;
+	char        *panel_id;
+	char        *pixmap;
+	char        *tooltip;
+
+	g_return_if_fail (panel_widget);
+	g_return_if_fail (gconf_key);
+
+	client  = panel_gconf_get_client ();
+
+	/* FIXME: Screen widths etc.. */
+	if (use_default)
+		profile = g_strdup ("medium");
+	else
+		profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "parameters", use_default);
+	panel = gconf_client_get_int (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "unique-drawer-panel-id", use_default);
+	panel_id = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "pixmap", use_default);
+	pixmap = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "tooltip", use_default);
+	tooltip = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	if (!panel_id < 0 && panel >= 0) {
+		PanelWidget *pw = g_slist_nth_data (panels, panel);
+
+		if (pw)
+			panel_id = g_strdup (pw->unique_id);
+	}
+
+	load_drawer_applet (panel_id, pixmap, tooltip, panel_widget, position, TRUE, gconf_key);
+
+	g_free (panel_id);
+	g_free (pixmap);
+	g_free (tooltip);
+	g_free (profile);
+}
