@@ -11,7 +11,7 @@
 #include "floating-widget.h"
 #include "border-widget.h"
 #include "panel-marshal.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
 #include "foobar-widget.h"
 #include "panel-util.h"
 #include "panel-gconf.h"
@@ -310,18 +310,24 @@ floating_pos_get_hide_size (BasePWidget *basep,
 			    PanelOrient hide_orient,
 			    int *w, int *h)
 {
+	gchar *panel_global_key;
+
 	if (basep->state == BASEP_AUTO_HIDDEN &&
 	    ! basep->hidebuttons_enabled) {
+		panel_global_key = panel_gconf_global_config_get_full_key ("panel-minimized-size");
+
 		switch (hide_orient) {
 		case PANEL_ORIENT_UP:
 		case PANEL_ORIENT_DOWN:
-			*h = panel_gconf_global_config_get_int ("panel-minimized-size");
+			*h = panel_gconf_get_int (panel_global_key, 3);
 			break;
 		case PANEL_ORIENT_LEFT:
 		case PANEL_ORIENT_RIGHT:
-			*w = panel_gconf_global_config_get_int ("panel-minimized-size");
+			*w = panel_gconf_get_int (panel_global_key, 3);
 			break;
 		}
+
+		g_free (panel_global_key);
 	} else {
 		switch (hide_orient) {
 		case PANEL_ORIENT_UP:
@@ -350,6 +356,10 @@ floating_pos_get_hide_pos (BasePWidget *basep,
 			   int *x, int *y,
 			   int w, int h)
 {
+	gchar *panel_global_key;
+
+	panel_global_key = panel_gconf_global_config_get_full_key ("panel-minimized-size");
+
 	switch (hide_orient) {
 	case PANEL_ORIENT_UP:
 	case PANEL_ORIENT_LEFT:
@@ -357,16 +367,18 @@ floating_pos_get_hide_pos (BasePWidget *basep,
 	case PANEL_ORIENT_RIGHT:
 		*x += w - ((basep->state == BASEP_AUTO_HIDDEN &&
 			    ! basep->hidebuttons_enabled)
-			   ? panel_gconf_global_config_get_int ("panel-minimized-size")
+			   ? panel_gconf_get_int (panel_global_key, 3)
 			   : get_requisition_width (basep->hidebutton_w));
 		break;
 	case PANEL_ORIENT_DOWN:
 		*y += h - ((basep->state == BASEP_AUTO_HIDDEN &&
 			    ! basep->hidebuttons_enabled)
-			   ? panel_gconf_global_config_get_int ("panel-minimized-size")
+			   ? panel_gconf_get_int (panel_global_key, 3)
 			   : get_requisition_height (basep->hidebutton_s));
 		break;
 	}
+	
+	g_free (panel_global_key);
 }
 
 static void
@@ -420,7 +432,7 @@ floating_widget_change_params (FloatingWidget *floating,
 			       PanelBackType back_type,
 			       char *back_pixmap,
 			       gboolean fit_pixmap_bg,
-			       gboolean strech_pixmap_bg,
+			       gboolean stretch_pixmap_bg,
 			       gboolean rotate_pixmap_bg,
 			       GdkColor *back_color)
 {
@@ -452,39 +464,12 @@ floating_widget_change_params (FloatingWidget *floating,
 				    back_type,
 				    back_pixmap,
 				    fit_pixmap_bg,
-				    strech_pixmap_bg,
+				    stretch_pixmap_bg,
 				    rotate_pixmap_bg,
 				    back_color);
 				    
 }
 
-void
-floating_widget_change_orient (FloatingWidget *floating,
-			       GtkOrientation orient)
-{
-	FloatingPos *pos = FLOATING_POS (floating->pos);
-	if (PANEL_WIDGET (BASEP_WIDGET (floating)->panel)->orient != orient) {
-		BasePWidget *basep = BASEP_WIDGET (floating);
-		PanelWidget *panel = PANEL_WIDGET (basep->panel);
-		floating_widget_change_params (floating, 
-					       basep->screen,
-					       pos->x,
-					       pos->y,
-					       orient,
-					       basep->mode,
-					       basep->state,
-					       panel->sz,
-					       basep->hidebuttons_enabled,
-					       basep->hidebutton_pixmaps_enabled,
-					       panel->back_type,
-					       panel->back_pixmap,
-					       panel->fit_pixmap_bg,
-					       panel->strech_pixmap_bg,
-					       panel->rotate_pixmap_bg,
-					       &panel->back_color);
-	}
-}
-
 void
 floating_widget_change_coords (FloatingWidget *floating,
 			       gint16 x, gint16 y)
@@ -506,14 +491,15 @@ floating_widget_change_coords (FloatingWidget *floating,
 					       panel->back_type,
 					       panel->back_pixmap,
 					       panel->fit_pixmap_bg,
-					       panel->strech_pixmap_bg,
+					       panel->stretch_pixmap_bg,
 					       panel->rotate_pixmap_bg,
 					       &panel->back_color);
 	}
 }
 
 GtkWidget *
-floating_widget_new (int screen,
+floating_widget_new (gchar *panel_id,
+		     int screen,
 		     gint16 x,
 		     gint16 y,
 		     GtkOrientation orient,
@@ -525,21 +511,22 @@ floating_widget_new (int screen,
 		     PanelBackType back_type,
 		     char *back_pixmap,
 		     gboolean fit_pixmap_bg,
-		     gboolean strech_pixmap_bg,
+		     gboolean stretch_pixmap_bg,
 		     gboolean rotate_pixmap_bg,
 		     GdkColor *back_color)
 {
 	FloatingWidget *floating;
 	FloatingPos *pos;
 
-	floating = gtk_type_new (FLOATING_TYPE_WIDGET);
-	floating->pos = gtk_type_new (FLOATING_TYPE_POS);
+	floating = g_object_new (FLOATING_TYPE_WIDGET, NULL);
+	floating->pos = g_object_new (FLOATING_TYPE_POS, NULL);
 	pos = FLOATING_POS (floating->pos);
 
 	pos->x = x;
 	pos->y = y;
 
-	basep_widget_construct (BASEP_WIDGET (floating),
+	basep_widget_construct (panel_id,
+				BASEP_WIDGET (floating),
 				TRUE, FALSE,
 				screen,
 				orient,
@@ -551,7 +538,7 @@ floating_widget_new (int screen,
 				back_type,
 				back_pixmap,
 				fit_pixmap_bg,
-				strech_pixmap_bg,
+				stretch_pixmap_bg,
 				rotate_pixmap_bg,
 				back_color);
 
