@@ -28,12 +28,6 @@
 
 #include "menu-properties.h"
 
-
-enum {
-	CLOSE_BUTTON,
-	HELP_BUTTON
-};
-
 #undef MENU_PROPERTIES_DEBUG
 
 struct _MenuDialogInfo {
@@ -41,8 +35,6 @@ struct _MenuDialogInfo {
 	GtkWidget *global_main;
 	GtkWidget *system;
 	GtkWidget *system_sub;
-	GtkWidget *user;
-	GtkWidget *user_sub;
 	GtkWidget *applets;
 	GtkWidget *applets_sub;
 	GtkWidget *distribution;
@@ -65,7 +57,7 @@ char *
 get_real_menu_path (const char *arguments, gboolean main_menu)
 {
 	if (main_menu)
-		return g_strdup ("programs:");
+		return g_strdup ("applications:");
 	
 	else if (*arguments == '~')
 		/* FIXME: this needs to be a URI */
@@ -97,7 +89,9 @@ get_pixmap (const char *menudir, gboolean main_menu)
 		g_free (dentry_name);
 
 		if (qitem != NULL)
-			pixmap_name = quick_desktop_item_find_icon (qitem->icon);
+			pixmap_name = gnome_desktop_item_find_icon (qitem->icon,
+								    20 /* desired size */,
+								    0 /* flags */);
 
 		if (pixmap_name == NULL)
 			pixmap_name = panel_pixmap_discovery ("gnome-folder.png",
@@ -109,29 +103,6 @@ get_pixmap (const char *menudir, gboolean main_menu)
 	return pixmap_name;
 }
 
-static gboolean
-got_gmenu (void)
-{
-	static gboolean checked = FALSE;
-	static gboolean got_it = FALSE;
-	char *tmp;
-
-	if (checked)
-		return got_it;
-
-	tmp = g_find_program_in_path  ("gmenu");
-	if (tmp != NULL)
-		got_it = TRUE;
-	else
-		got_it = FALSE;
-
-	g_free (tmp);
-
-	checked = TRUE;
-
-	return got_it;
-}
-
 static void
 properties_apply_callback (Menu *menu)
 {
@@ -178,9 +149,7 @@ properties_apply_callback (Menu *menu)
 	    GTK_TOGGLE_BUTTON (menu->dialog_info->global_main)->active) {
 		menu->main_menu = TRUE;
 
-		if (got_gmenu ()) {
-			need_edit_menus = TRUE;
-		}
+		need_edit_menus = TRUE;
 	} else {
 		s = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (menu->dialog_info->pathentry),
 						    TRUE);
@@ -223,11 +192,6 @@ properties_apply_callback (Menu *menu)
 	else if (GTK_TOGGLE_BUTTON(menu->dialog_info->system)->active)
 		menu->main_menu_flags |= MAIN_MENU_SYSTEM;
 
-	if(GTK_TOGGLE_BUTTON(menu->dialog_info->user_sub)->active)
-		menu->main_menu_flags |= MAIN_MENU_USER_SUB;
-	else if (GTK_TOGGLE_BUTTON (menu->dialog_info->user)->active)
-		menu->main_menu_flags |= MAIN_MENU_USER;
-
 	if(GTK_TOGGLE_BUTTON(menu->dialog_info->applets_sub)->active)
 		menu->main_menu_flags |= MAIN_MENU_APPLETS_SUB;
 	else if (GTK_TOGGLE_BUTTON (menu->dialog_info->applets)->active)
@@ -398,9 +362,9 @@ add_menu_type_options(Menu *menu, GtkObject *dialog, GtkTable *table, int row,
 			  G_CALLBACK (toggle_prop), 
 			  menu);
 	
-	w = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(rb)),
+	w = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb)),
 					     _("In a submenu"));
-	gtk_table_attach_defaults(table, w, 2, 3, row, row+1);
+	gtk_table_attach_defaults (table, w, 2, 3, row, row+1);
 
 	*widget_sub = w;
 
@@ -410,7 +374,7 @@ add_menu_type_options(Menu *menu, GtkObject *dialog, GtkTable *table, int row,
 			  G_CALLBACK (toggle_prop), 
 			  menu);
 	
-	w = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(rb)),
+	w = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb)),
 					     _("On the main menu"));
 	*widget = w;
 	gtk_table_attach_defaults(table, w, 1, 2, row, row+1);
@@ -426,9 +390,9 @@ dialog_response (GtkWidget *dialog, int response, gpointer data)
 {
 	Menu *menu = data;
 
-	if (response == CLOSE_BUTTON) {
+	if (response == GTK_RESPONSE_CLOSE) {
 		gtk_widget_destroy (dialog);
-	} else if (response == HELP_BUTTON) {
+	} else if (response == GTK_RESPONSE_HELP) {
 		if (GTK_TOGGLE_BUTTON (menu->dialog_info->main_menu)->active)
 			panel_show_help ("mainmenu", "MAINMENUCONFIG");
 		else
@@ -451,10 +415,10 @@ create_properties_dialog (Menu *menu)
 	dialog = gtk_dialog_new_with_buttons (_("Menu properties"),
 					      NULL /* parent */,
 					      0 /* flags */,
-					      GTK_STOCK_CLOSE,
-					      CLOSE_BUTTON,
 					      GTK_STOCK_HELP,
-					      HELP_BUTTON,
+					      GTK_RESPONSE_HELP,
+					      GTK_STOCK_CLOSE,
+					      GTK_RESPONSE_CLOSE,
 					      NULL);
 
 	menu->prop_dialog = dialog;
@@ -467,8 +431,6 @@ create_properties_dialog (Menu *menu)
 
 	gtk_window_set_wmclass(GTK_WINDOW(dialog),
 			       "menu_properties", "Panel");
-	/*gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);*/
-	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
 	vbox = gtk_vbox_new(FALSE,GNOME_PAD_SMALL);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox),GNOME_PAD_SMALL);
@@ -486,29 +448,32 @@ create_properties_dialog (Menu *menu)
 	if (menu->main_menu &&
 	    menu->global_main)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+
 	g_signal_connect (G_OBJECT (w), "toggled", 
 			  G_CALLBACK (toggle_global_main), 
 			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
 
 	w = gtk_radio_button_new_with_label (
-		  gtk_radio_button_group (GTK_RADIO_BUTTON (global_main)),
+		  gtk_radio_button_get_group (GTK_RADIO_BUTTON (global_main)),
 		  _("Main menu"));
 	main_menu = w;
 	menu->dialog_info->main_menu = w;
 	if (menu->main_menu &&
 	    ! menu->global_main)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+
 	g_signal_connect (G_OBJECT (w), "toggled", 
 			  G_CALLBACK (toggle_main_menu), 
 			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
 
 	w2 = gtk_radio_button_new_with_label (
-		  gtk_radio_button_group (GTK_RADIO_BUTTON (global_main)),
+		  gtk_radio_button_get_group (GTK_RADIO_BUTTON (global_main)),
 		  _("Normal menu"));
 	if ( ! menu->main_menu)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w2), TRUE);
+
 	g_signal_connect (G_OBJECT (w2), "toggled", 
 			  G_CALLBACK (toggle_normal_menu), 
 			  menu);
@@ -532,13 +497,6 @@ create_properties_dialog (Menu *menu)
 			      &menu->dialog_info->system_sub,
 			      menu->main_menu_flags & MAIN_MENU_SYSTEM,
 			      menu->main_menu_flags & MAIN_MENU_SYSTEM_SUB);
-	add_menu_type_options(menu,
-			      GTK_OBJECT(dialog), GTK_TABLE(table),1,
-			      _("Favorites: "),
-			      &menu->dialog_info->user,
-			      &menu->dialog_info->user_sub,
-			      menu->main_menu_flags & MAIN_MENU_USER,
-			      menu->main_menu_flags & MAIN_MENU_USER_SUB);
 	add_menu_type_options(menu,
 			      GTK_OBJECT(dialog), GTK_TABLE(table),2,
 			      _("Applets: "),
@@ -653,6 +611,20 @@ create_properties_dialog (Menu *menu)
 			  G_CALLBACK (dialog_response),
 			  menu);
 
+	/* Set the sensitivity of the frames as req'd. */
+	if (menu->main_menu) {
+		if (menu->global_main) {
+			gtk_widget_set_sensitive(menu->dialog_info->main_frame, FALSE);
+			gtk_widget_set_sensitive(menu->dialog_info->normal_frame, FALSE);
+		} else {
+			gtk_widget_set_sensitive(menu->dialog_info->main_frame, TRUE);
+			gtk_widget_set_sensitive(menu->dialog_info->normal_frame, FALSE);
+		}
+	} else {
+		gtk_widget_set_sensitive(menu->dialog_info->main_frame, FALSE);
+		gtk_widget_set_sensitive(menu->dialog_info->normal_frame, TRUE);
+	}
+
 	return dialog;
 }
 
@@ -664,8 +636,7 @@ menu_properties (Menu *menu)
 	g_return_if_fail (menu != NULL);
 
 	if (menu->prop_dialog != NULL) {
-		gtk_widget_show_now (menu->prop_dialog);
-		gdk_window_raise (menu->prop_dialog->window);
+		gtk_window_present (GTK_WINDOW (menu->prop_dialog));
 		return;
 	}
 
