@@ -21,10 +21,6 @@
  */
 
 #include <config.h>
-#include <gnome.h>
-
-#include "panel-include.h"
-#include "foobar-widget.h"
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -35,6 +31,9 @@
 
 #include "multiscreen-stuff.h"
 
+#include "foobar-widget.h"
+#include "basep-widget.h"
+
 /* some globals */
 static int		screens = 1;
 static GdkRectangle *	rectangles = NULL;
@@ -76,9 +75,6 @@ multiscreen_init (void)
 	if (gdk_error_trap_pop () != 0)
 		have_xinerama = FALSE;
 
-	if (g_getenv ("GNOME_PANEL_NO_XINERAMA") != NULL)
-		have_xinerama = FALSE;
-
 	if (have_xinerama) {
 		int screen_num, i;
 		XineramaScreenInfo *xscreens =
@@ -132,52 +128,48 @@ int
 multiscreen_x (int screen)
 {
 	g_return_val_if_fail (initialized, 0);
+	g_return_val_if_fail (screen >= 0, 0);
 
-	/* If we don't know what screen we're talking about
-	 * assume screen 0 */
-	if (screen < 0 || screen >= screens)
-		screen = 0;
-
-	return rectangles[screen].x;
+	if (screen < screens)
+		return rectangles[screen].x;
+	else
+		return gdk_screen_width () + 10;
 }
 
 int
 multiscreen_y (int screen)
 {
 	g_return_val_if_fail (initialized, 0);
+	g_return_val_if_fail (screen >= 0, 0);
 
-	/* If we don't know what screen we're talking about
-	 * assume screen 0 */
-	if (screen < 0 || screen >= screens)
-		screen = 0;
-
-	return rectangles[screen].y;
+	if (screen < screens)
+		return rectangles[screen].y;
+	else
+		return gdk_screen_height () + 10;
 }
 
 int
 multiscreen_width (int screen)
 {
 	g_return_val_if_fail (initialized, 0);
+	g_return_val_if_fail (screen >= 0, 0);
 
-	/* If we don't know what screen we're talking about
-	 * assume screen 0 */
-	if (screen < 0 || screen >= screens)
-		screen = 0;
-
-	return rectangles[screen].width;
+	if (screen < screens)
+		return rectangles[screen].width;
+	else
+		return gdk_screen_width ();
 }
 
 int
 multiscreen_height (int screen)
 {
 	g_return_val_if_fail (initialized, 0);
+	g_return_val_if_fail (screen >= 0, 0);
 
-	/* If we don't know what screen we're talking about
-	 * assume screen 0 */
-	if (screen < 0 || screen >= screens)
-		screen = 0;
-
-	return rectangles[screen].height;
+	if (screen < screens)
+		return rectangles[screen].height;
+	else
+		return gdk_screen_height ();
 }
 
 int
@@ -186,9 +178,9 @@ multiscreen_screen_from_pos (int x, int y)
 	int i;
 	for (i = 0; i < screens; i++) {
 		if (x >= rectangles[i].x &&
-		    x < rectangles[i].x + rectangles[i].width &&
+		    x <= rectangles[i].x + rectangles[i].width &&
 		    y >= rectangles[i].y &&
-		    y < rectangles[i].y + rectangles[i].height)
+		    y <= rectangles[i].y + rectangles[i].height)
 			return i;
 	}
 	return -1;
@@ -198,12 +190,12 @@ int
 multiscreen_screen_from_panel (GtkWidget *widget)
 {
 	g_return_val_if_fail (widget != NULL, 0);
-	g_return_val_if_fail (IS_BASEP_WIDGET (widget) ||
-			      IS_FOOBAR_WIDGET (widget), 0);
+	g_return_val_if_fail (BASEP_IS_WIDGET (widget) ||
+			      FOOBAR_IS_WIDGET (widget), 0);
 	
-	if (IS_BASEP_WIDGET (widget))
+	if (BASEP_IS_WIDGET (widget))
 		return BASEP_WIDGET (widget)->screen;
-	else if (IS_FOOBAR_WIDGET (widget))
+	else if (FOOBAR_IS_WIDGET (widget))
 		return FOOBAR_WIDGET (widget)->screen;
 	else
 		return 0;
