@@ -16,10 +16,16 @@
 #include <fcntl.h>
 #include <dirent.h>
 
+/* includes for the battery stuff */
+#include <signal.h>
+#include <limits.h>
+#include <unistd.h>
+#include <sys/wait.h>
+
 #include <libgnomeui/libgnomeui.h>
 #include <libgnome/libgnome.h>
 
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -27,7 +33,8 @@
 #include "panel-util.h"
 
 #include "applet.h"
-#include "panel_config_global.h"
+#include "nothing.h"
+#include "panel-config-global.h"
 
 extern GlobalConfig global_config;
 
@@ -128,19 +135,10 @@ panel_gnome_kde_help_path (const char *docpath)
 #endif
 }
 
-gboolean
-string_is_in_list(const GSList *list, const char *text)
-{
-	for(;list != NULL; list = list->next)
-		if(strcmp(text,(char *)list->data) == 0)
-			return TRUE;
-	return FALSE;
-}
-
 static void
 updated (GtkWidget *w, gpointer data)
 {
-	UpdateFunction func = gtk_object_get_data (GTK_OBJECT (w), "update_function");
+	UpdateFunction func = g_object_get_data (G_OBJECT (w), "update_function");
 
 	func (data);
 }
@@ -178,7 +176,7 @@ create_text_entry(GtkWidget *table,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	if(func) {
-		gtk_object_set_data (GTK_OBJECT (t), "update_function", func);
+		g_object_set_data (G_OBJECT (t), "update_function", func);
 		g_signal_connect (G_OBJECT (t), "changed",
 				  G_CALLBACK (updated), 
 				  data);
@@ -221,7 +219,7 @@ create_icon_entry(GtkWidget *table,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	if(func) {
-		gtk_object_set_data (GTK_OBJECT (entry), "update_function", func);
+		g_object_set_data (G_OBJECT (entry), "update_function", func);
 		g_signal_connect (G_OBJECT (entry), "changed",
 				  G_CALLBACK (updated), 
 				  data);
@@ -493,7 +491,7 @@ set_color_back (GtkWidget *widget, PanelWidget *panel)
 	GtkStyle *ns;
 	int i;
 
-	gtk_widget_set_rc_style (widget);
+	gtk_widget_set_style (widget, NULL);
 	ns = gtk_style_copy (gtk_widget_get_style (widget));
 
 	ns->bg[GTK_STATE_NORMAL] =
@@ -514,7 +512,7 @@ set_color_back (GtkWidget *widget, PanelWidget *panel)
 		ns->mid[i].blue = (ns->light[i].blue + ns->dark[i].blue) / 2;
 	}
 	gtk_widget_set_style (widget, ns);
-	gtk_style_unref (ns);
+	g_object_unref (G_OBJECT (ns));
 }
 
 void
@@ -529,11 +527,11 @@ set_frame_colors (PanelWidget *panel, GtkWidget *frame,
 		set_color_back (but3, panel);
 		set_color_back (but4, panel);
 	} else {
-		gtk_widget_set_rc_style (frame);
-		gtk_widget_set_rc_style (but1);
-		gtk_widget_set_rc_style (but2);
-		gtk_widget_set_rc_style (but3);
-		gtk_widget_set_rc_style (but4);
+		gtk_widget_set_style (frame, NULL);
+		gtk_widget_set_style (but1, NULL);
+		gtk_widget_set_style (but2, NULL);
+		gtk_widget_set_style (but3, NULL);
+		gtk_widget_set_style (but4, NULL);
 	}
 }
 
@@ -567,80 +565,10 @@ remove_directory(const char *dirname, gboolean just_clean)
 	g_free(oldcwd);
 }
 
-char *
-strtok_with_escape(char *string, const char *seps, gboolean empty)
-{
-	char *our_string;
-	static char *p = NULL;
-
-	g_return_val_if_fail(seps != NULL, NULL);
-
-	if(string)
-		p = string;
-	else if(!p)
-		return NULL;
-
-	our_string = p;
-	while(*p) {
-		if(*p == '\\' && *(p+1)) {
-			strcpy(p,p+1);
-		} else if(strchr(seps,*p)) {
-			*p = '\0';
-			if(empty || *our_string) {
-				p++;
-				return our_string;
-			}
-
-			/* this was an empty part and
-			   we don't want to return empty
-			   parts, so skip it */
-			our_string = p+1;
-		}
-		p++;
-	}
-	p = NULL;
-
-	if(!empty && !*our_string)
-		return NULL;
-	return our_string;
-}
-
-/* return a newly allocated string that escapes / and 'special' */
-char *
-escape_string(const char *string, const char *special)
-{
-	int count;
-	const char *p;
-	char *ret;
-	int i;
-
-	if(!string) return NULL;
-
-	g_return_val_if_fail(special != NULL, NULL);
-
-	for(p=string,count=0; *p; p++,count++) {
-		if(*p == '\\' ||
-		   strchr(special,*p))
-			count++;
-	}
-
-	ret = g_new(char,count+1);
-	i = 0;
-	for(p=string; *p; p++) {
-		if(*p == '\\' ||
-		   strchr(special,*p))
-			ret[i++] = '\\';
-		ret[i++] = *p;
-	}
-	ret[i] = '\0';
-
-	return ret;
-}
-
 gboolean
 convert_string_to_keysym_state(const char *string,
 			       guint *keysym,
-			       guint *state)
+			       GdkModifierType *state)
 {
 	char *s, *p;
 
@@ -655,44 +583,7 @@ convert_string_to_keysym_state(const char *string,
 	   strcmp (string, _("Disabled")) == 0)
 		return FALSE;
 
-	s = g_strdup (string);
-
-	gdk_error_trap_push ();
-
-	p = strtok (s, "-");
-	while (p != NULL) {
-		if(strcmp(p, "Control")==0) {
-			*state |= GDK_CONTROL_MASK;
-		} else if(strcmp(p, "Lock")==0) {
-			*state |= GDK_LOCK_MASK;
-		} else if(strcmp(p, "Shift")==0) {
-			*state |= GDK_SHIFT_MASK;
-		} else if(strcmp(p, "Mod1")==0) {
-			*state |= GDK_MOD1_MASK;
-		} else if(strcmp(p, "Mod2")==0) {
-			*state |= GDK_MOD2_MASK;
-		} else if(strcmp(p, "Mod3")==0) {
-			*state |= GDK_MOD3_MASK;
-		} else if(strcmp(p, "Mod4")==0) {
-			*state |= GDK_MOD4_MASK;
-		} else if(strcmp(p, "Mod5")==0) {
-			*state |= GDK_MOD5_MASK;
-		} else {
-			*keysym = gdk_keyval_from_name(p);
-			if(*keysym == 0) {
-				gdk_flush();
-				gdk_error_trap_pop();
-				g_free(s);
-				return FALSE;
-			}
-		} 
-		p = strtok(NULL, "-");
-	}
-
-	gdk_flush ();
-	gdk_error_trap_pop ();
-
-	g_free (s);
+	gtk_accelerator_parse (string, keysym, state);
 
 	if (*keysym == 0)
 		return FALSE;
@@ -702,7 +593,7 @@ convert_string_to_keysym_state(const char *string,
 
 char *
 convert_keysym_state_to_string(guint keysym,
-			       guint state)
+			       GdkModifierType state)
 {
 	GString *gs;
 	char *sep = "";
@@ -711,135 +602,7 @@ convert_keysym_state_to_string(guint keysym,
 	if(keysym == 0)
 		return g_strdup(_("Disabled"));
 
-	gdk_error_trap_push();
-	key = gdk_keyval_name(keysym);
-	gdk_flush();
-	gdk_error_trap_pop();
-	if(!key) return NULL;
-
-	gs = g_string_new(NULL);
-
-	if(state & GDK_CONTROL_MASK) {
-		/*g_string_append(gs, sep);*/
-		g_string_append(gs, "Control");
-		sep = "-";
-	}
-	if(state & GDK_LOCK_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Lock");
-		sep = "-";
-	}
-	if(state & GDK_SHIFT_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Shift");
-		sep = "-";
-	}
-	if(state & GDK_MOD1_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod1");
-		sep = "-";
-	}
-	if(state & GDK_MOD2_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod2");
-		sep = "-";
-	}
-	if(state & GDK_MOD3_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod3");
-		sep = "-";
-	}
-	if(state & GDK_MOD4_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod4");
-		sep = "-";
-	}
-	if(state & GDK_MOD5_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod5");
-		sep = "-";
-	}
-
-	g_string_append(gs, sep);
-	g_string_append(gs, key);
-
-	{
-		char *ret = gs->str;
-		g_string_free(gs, FALSE);
-		return ret;
-	}
-}
-
-#if FIXME
-static GSList *layered_dialogs = NULL;
-#endif /* FIXME */
-
-#if FIXME
-static void
-panel_dialog_realized (GtkWidget *dialog)
-{
-	if ( ! global_config.keep_bottom &&
-	     ! global_config.normal_layer)
-		gnome_win_hints_set_layer (GTK_WIDGET(dialog),
-					   WIN_LAYER_ABOVE_DOCK);
-	else
-		gnome_win_hints_set_layer (GTK_WIDGET (dialog),
-					   WIN_LAYER_NORMAL);
-}
-#endif /* FIXME */
-
-#if FIXME
-static void
-remove_from_layered (GtkWidget *w)
-{
-	layered_dialogs = g_slist_remove (layered_dialogs, w);
-}
-#endif /* FIXME */
-
-void
-panel_set_dialog_layer (GtkWidget *dialog)
-{
-#if FIXME
-	if (g_slist_find (layered_dialogs, dialog) == NULL) {
-		layered_dialogs = g_slist_prepend (layered_dialogs, dialog);
-		g_signal_connect (G_OBJECT (dialog), "destroy",
-				  G_CALLBACK (remove_from_layered),
-				  NULL);
-	}
-
-	if (GTK_WIDGET_REALIZED (dialog) &&
-	    ! global_config.normal_layer &&
-	    ! global_config.keep_bottom)
-		gnome_win_hints_set_layer (GTK_WIDGET (dialog),
-					   WIN_LAYER_ABOVE_DOCK);
-
-	g_signal_connect (G_OBJECT (dialog), "realize",
-			  G_CALLBACK (panel_dialog_realized),
-			  NULL);
-#endif
-}
-
-void
-panel_reset_dialog_layers (void)
-{
-#if FIXME
-	GSList *li;
-
-	for (li = layered_dialogs; li != NULL; li = li->next) {
-		GtkWidget *dialog = li->data;
-
-		if ( ! GTK_WIDGET_REALIZED (dialog))
-			continue;
-
-		if ( ! global_config.normal_layer &&
-		     ! global_config.keep_bottom)
-			gnome_win_hints_set_layer (GTK_WIDGET (dialog),
-						   WIN_LAYER_ABOVE_DOCK);
-		else
-			gnome_win_hints_set_layer (GTK_WIDGET (dialog),
-						   WIN_LAYER_NORMAL);
-	}
-#endif
+	return gtk_accelerator_name (keysym, state);
 }
 
 GtkWidget *
@@ -862,12 +625,14 @@ panel_error_dialog (const char *class,
 
 	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_OK, s);
+	gtk_widget_add_events (w, GDK_KEY_PRESS_MASK);
+	g_signal_connect (G_OBJECT (w), "event",
+			  G_CALLBACK (panel_dialog_window_event), NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (w),
 				class, "Panel");
 	g_free (s);
 
 	gtk_widget_show_all (w);
-	panel_set_dialog_layer (w);
 
 	g_signal_connect_swapped (G_OBJECT (w), "response",
 				   G_CALLBACK (gtk_widget_destroy),
@@ -896,12 +661,13 @@ panel_info_dialog (const char *class,
 
 	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
 				    GTK_BUTTONS_OK, s);
+	g_signal_connect (G_OBJECT (w), "event",
+			  G_CALLBACK (panel_dialog_window_event), NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (w),
 				class, "Panel");
 	g_free (s);
 
 	gtk_widget_show_all (w);
-	panel_set_dialog_layer (w);
 
 	g_signal_connect_swapped (G_OBJECT (w), "response",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -1011,57 +777,11 @@ panel_is_url (const char *url)
 		return FALSE;
 }
 
-static gboolean
-no_need_to_quote (const char *str)
-{
-	const char *p;
-	
-	/* empty string needs quoting */
-	if (string_empty (str))
-		return TRUE;
-
-	for (p = str; *p != '\0'; p++) {
-		if ((*p >= 'a' && *p <= 'z') ||
-		    (*p >= 'A' && *p <= 'Z') ||
-		    strchr ("_-./", *p) != NULL)
-			continue;
-
-		return FALSE;
-	}
-	return TRUE;
-}
-
-/* Quote string so that it could be on a shell like
- * quoted line as an argument */
-char *
-panel_quote_string (const char *str)
-{
-	GString *gs;
-	const char *p;
-
-	if (string_empty (str))
-		return g_strdup ("''");
-
-	if (no_need_to_quote (str))
-		return g_strdup (str);
-
-	gs = g_string_new ("'");
-	for (p = str; *p != '\0'; p++) {
-		if (*p == '\'')
-			g_string_append (gs, "'\\''");
-		else
-			g_string_append_c (gs, *p);
-	}
-	g_string_append_c (gs, '\'');
-
-	return g_string_free (gs, FALSE);
-}
-
 void
 panel_push_window_busy (GtkWidget *window)
 {
-	int busy = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (window),
-							 "Panel:WindowBusy"));
+	int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window),
+						       "Panel:WindowBusy"));
 
 	busy ++;
 
@@ -1070,31 +790,31 @@ panel_push_window_busy (GtkWidget *window)
 		if (window->window != NULL) {
 			GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
 			gdk_window_set_cursor (window->window, cursor);
-			gdk_cursor_destroy (cursor);
+			gdk_cursor_unref (cursor);
 			gdk_flush ();
 		}
 	}
 
-	gtk_object_set_data (GTK_OBJECT (window), "Panel:WindowBusy",
-			     GINT_TO_POINTER (busy));
+	g_object_set_data (G_OBJECT (window), "Panel:WindowBusy",
+			   GINT_TO_POINTER (busy));
 }
 
 void
 panel_pop_window_busy (GtkWidget *window)
 {
-	int busy = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (window),
-							 "Panel:WindowBusy"));
+	int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window),
+						       "Panel:WindowBusy"));
 	busy --;
 
 	if (busy <= 0) {
 		gtk_widget_set_sensitive (window, TRUE);
 		if (window->window != NULL)
 			gdk_window_set_cursor (window->window, NULL);
-		gtk_object_remove_data (GTK_OBJECT (window),
-					"Panel:WindowBusy");
+		g_object_set_data (G_OBJECT (window),
+				   "Panel:WindowBusy", NULL);
 	} else {
-		gtk_object_set_data (GTK_OBJECT (window), "Panel:WindowBusy",
-				     GINT_TO_POINTER (busy));
+		g_object_set_data (G_OBJECT (window), "Panel:WindowBusy",
+				   GINT_TO_POINTER (busy));
 	}
 }
 
@@ -1252,3 +972,356 @@ panel_pixmap_discovery (const char *name, gboolean fallback)
 						    NULL /* ret_locations */);
 	return pixmap;
 }
+
+typedef struct {
+	gboolean top;
+	gboolean right;
+	gboolean bottom;
+	gboolean left;
+} StrechSides;
+
+static void
+stretch_widget_realize (GtkWidget *widget)
+{
+	GdkWindowAttr attributes = {0};
+	int attributes_mask;
+	GtkWidget *toplevel;
+	int x,y,w,h;
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
+						 "StrechEventWindow");
+	StrechSides *sides = g_object_get_data (G_OBJECT (widget),
+						"StrechSides");
+	if (sides == NULL)
+		return;
+
+	if (eventwin != NULL)
+		gdk_window_destroy (eventwin);
+
+	toplevel = gtk_widget_get_toplevel (widget);
+
+	if (toplevel == NULL ||
+	    ! GTK_WIDGET_REALIZED (toplevel))
+		return;
+	
+	gtk_widget_translate_coordinates (widget /* src_widget */,
+					  toplevel /* dest_widget */,
+					  0, 0 /* src */,
+					  &x, &y /* dest */);
+
+	w = widget->allocation.width;
+	h = widget->allocation.height;
+
+	if (sides->top) {
+		h += y;
+		y = 0;
+	}
+	if (sides->left) {
+		w += x;
+		x = 0;
+	}
+	if (sides->bottom)
+		h = toplevel->allocation.height - y;
+	if (sides->right)
+		w = toplevel->allocation.width - x;
+
+	attributes.window_type = GDK_WINDOW_CHILD;
+	attributes.x = x;
+	attributes.y = y;
+	attributes.width = w;
+	attributes.height = h;
+	attributes.wclass = GDK_INPUT_ONLY;
+	attributes.event_mask = (GDK_BUTTON_PRESS_MASK |
+				 GDK_BUTTON_RELEASE_MASK |
+				 GDK_POINTER_MOTION_MASK |
+				 GDK_POINTER_MOTION_HINT_MASK |
+				 GDK_KEY_PRESS_MASK |
+				 GDK_ENTER_NOTIFY_MASK |
+				 GDK_LEAVE_NOTIFY_MASK);
+	attributes_mask = GDK_WA_X | GDK_WA_Y;
+
+	eventwin = gdk_window_new (toplevel->window,
+				   &attributes,
+				   attributes_mask);
+	gdk_window_set_user_data (eventwin, widget);
+
+	g_object_set_data (G_OBJECT (widget),
+			   "StrechEventWindow",
+			   eventwin);
+}
+
+static void
+stretch_widget_unrealize (GtkWidget *widget)
+{
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
+						 "StrechEventWindow");
+	if (eventwin == NULL)
+		return;
+
+	gdk_window_destroy (eventwin);
+	g_object_set_data (G_OBJECT (widget),
+			   "StrechEventWindow",
+			   NULL);
+}
+
+/* Evil but otherwise it doesn't seem
+ * to work.  There needs to be a cleaner
+ * solution */
+static gboolean
+raise_in_idle (gpointer data)
+{
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (data),
+						 "StrechEventWindow");
+
+	g_object_unref (G_OBJECT (data));
+
+	if (eventwin == NULL)
+		return FALSE;
+
+	gdk_window_raise (eventwin);
+	return FALSE;
+}
+
+
+static void
+stretch_widget_map (GtkWidget *widget)
+{
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
+						 "StrechEventWindow");
+	if (eventwin == NULL)
+		return;
+
+	if (GTK_WIDGET_MAPPED (widget)) {
+		gdk_window_show (eventwin);
+		gdk_window_raise (eventwin);
+		g_idle_add (raise_in_idle,
+			    g_object_ref (G_OBJECT (widget)));
+	}
+}
+
+static void
+stretch_widget_unmap (GtkWidget *widget)
+{
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
+						 "StrechEventWindow");
+	if (eventwin == NULL)
+		return;
+
+	gdk_window_hide (eventwin);
+}
+
+
+static void
+stretch_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+{
+	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
+						 "StrechEventWindow");
+	if (eventwin == NULL)
+		return;
+
+	/* somewhat evil */
+	stretch_widget_unrealize (widget);
+	stretch_widget_realize (widget);
+	stretch_widget_map (widget);
+}
+
+static void
+stretch_widget_hierarchy_changed (GtkWidget *widget,
+				 GtkWidget *previous_toplevel)
+{
+	stretch_widget_unrealize (widget);
+	stretch_widget_realize (widget);
+	stretch_widget_map (widget);
+}
+
+void
+panel_stretch_events_to_toplevel (GtkWidget *widget,
+				 gboolean top,
+				 gboolean right,
+				 gboolean bottom,
+				 gboolean left)
+{
+	StrechSides *sides;
+
+	g_signal_connect_after (GTK_WIDGET (widget), "realize",
+				G_CALLBACK (stretch_widget_realize),
+				NULL);
+	g_signal_connect (GTK_WIDGET (widget), "unrealize",
+			  G_CALLBACK (stretch_widget_unrealize),
+			  NULL);
+	g_signal_connect_after (GTK_WIDGET (widget), "size_allocate",
+				G_CALLBACK (stretch_widget_size_allocate),
+				NULL);
+	g_signal_connect_after (GTK_WIDGET (widget), "map",
+				G_CALLBACK (stretch_widget_map),
+				NULL);
+	g_signal_connect_after (GTK_WIDGET (widget), "unmap",
+				G_CALLBACK (stretch_widget_unmap),
+				NULL);
+	g_signal_connect_after (GTK_WIDGET (widget), "hierarchy_changed",
+				G_CALLBACK (stretch_widget_hierarchy_changed),
+				NULL);
+
+	sides = g_new0 (StrechSides, 1);
+	sides->top = top;
+	sides->right = right;
+	sides->bottom = bottom;
+	sides->left = left;
+
+	g_object_set_data_full (G_OBJECT (widget),
+				"StrechSides",
+				sides,
+				(GDestroyNotify) g_free);
+}
+
+/* stolen from gtk */
+void
+panel_signal_connect_while_alive (GObject     *object,
+				  const gchar *signal,
+				  GCallback    func,
+				  gpointer     func_data,
+				  GObject     *alive_object)
+{
+	GClosure *closure;
+
+	g_return_if_fail (G_IS_OBJECT (object));
+
+	closure = g_cclosure_new (func, func_data, NULL);
+	g_object_watch_closure (G_OBJECT (alive_object), closure);
+	g_signal_connect_closure_by_id (object,
+					g_signal_lookup (signal, G_OBJECT_TYPE (object)), 0,
+					closure,
+					FALSE);
+}
+
+void
+panel_signal_connect_object_while_alive (GObject      *object,
+					 const gchar  *signal,
+					 GCallback     func,
+					 GObject      *alive_object)
+{
+  g_return_if_fail (G_IS_OBJECT (object));
+  
+  g_signal_connect_closure_by_id (object,
+				  g_signal_lookup (signal, G_OBJECT_TYPE (object)), 0,
+				  g_cclosure_new_object_swap (func, G_OBJECT (alive_object)),
+				  FALSE);
+}
+
+gboolean
+panel_ensure_dir (const char *dirname)
+{
+	char *parsed, *p;
+
+	if (dirname == NULL)
+		return FALSE;
+
+	parsed = g_strdup (dirname);
+
+	if (g_file_test (parsed, G_FILE_TEST_IS_DIR)) {
+		g_free (parsed);
+		return TRUE;
+	}
+
+	p = strchr (parsed, '/');
+	if (p == parsed)
+		p = strchr (p+1, '/');
+
+	while (p != NULL) {
+		*p = '\0';
+		if (mkdir (parsed, 0700) != 0 &&
+		    errno != EEXIST) {
+			g_free (parsed);
+			return FALSE;
+		}
+		*p = '/';
+		p = strchr (p+1, '/');
+	}
+
+	if (mkdir (parsed, 0700) != 0 &&
+	    errno != EEXIST) {
+		g_free (parsed);
+		return FALSE;
+	}
+
+	g_free (parsed);
+	return TRUE;
+}
+
+#if 0
+/* hmmm, we need to use this I think to add a battery applet
+ *  -George */
+/* Linux Battery Stuff */
+
+static gboolean
+linux_battery_exists (void)
+{
+        FILE *fp;
+        char buf[200] = "";
+        int foo;
+
+        if ( ! g_file_test ("/proc/apm", G_FILE_TEST_EXISTS))
+                return FALSE;
+
+        fp = fopen ("/proc/apm", "r");
+        if (fp == NULL)
+                return FALSE;
+
+        if (fgets (buf, sizeof (buf), fp) == NULL) {
+                fclose (fp);
+                return FALSE;
+        }
+        fclose (fp);
+
+        foo = -1;
+        sscanf (buf,
+                "%*s %*d.%*d %*x %*x %*x %*x %d%% %*d %*s\n",
+                &foo);
+
+        if (foo >= 0)
+                return TRUE;
+        else
+                return FALSE;
+}
+
+static gboolean
+battery_exists (void)
+{
+#ifndef __linux__
+        return FALSE;
+#else
+        /* This is MUUUUCHO ugly, but apparently RH 7.0 with segv the program
+         * reading /proc/apm on some laptops, and we don't want to crash, thus
+         * we do the check in a forked process */
+        int status;
+        pid_t pid;
+
+        pid = fork ();
+        if (pid == 0) {
+                struct sigaction sa = {{NULL}};
+
+                sa.sa_handler = SIG_DFL;
+
+                sigaction(SIGSEGV, &sa, NULL);
+                sigaction(SIGFPE, &sa, NULL);
+                sigaction(SIGBUS, &sa, NULL);
+
+                if (linux_battery_exists ())
+                        _exit (0);
+                else
+                        _exit (1);
+        }
+
+        status = 0;
+        waitpid (pid, &status, 0);
+
+        if ( ! WIFSIGNALED (status) &&
+            WIFEXITED (status) &&
+            WEXITSTATUS (status) == 0) {
+                return TRUE;
+        } else {
+                return FALSE;
+        }
+#endif
+}
+
+#endif
