@@ -10,9 +10,6 @@ G_BEGIN_DECLS
 
 typedef void (*UpdateFunction) (gpointer);
 
-gboolean	string_is_in_list	(const GSList *list,
-					 const char *text);
-
 /* return TRUE if string is NULL or the first character is '\0' */
 gboolean	string_empty		(const char *string);
 #define		sure_string(s)		((s)!=NULL?(s):"")
@@ -61,23 +58,12 @@ void		set_frame_colors	(PanelWidget *panel,
 void		remove_directory	(const char *dirname,
 					 gboolean just_clean);
 
-/* like strtok, but unescapes string as well and escaped separators
-   don't count.  if empty is true, don't skip empty parts */
-char *		strtok_with_escape	(char *string,
-					 const char *seps,
-					 gboolean empty);
-/* return a newly allocated string that escapes / and 'special' */
-char *		escape_string		(const char *string,
-					 const char *special);
-
 gboolean	convert_string_to_keysym_state (const char *string,
 						guint *keysym,
-						guint *state);
+						GdkModifierType *state);
 char *		convert_keysym_state_to_string (guint keysym,
-						guint state);
+						GdkModifierType state);
 
-void		panel_set_dialog_layer	(GtkWidget *dialog);
-void		panel_reset_dialog_layers (void);
 GtkWidget *	panel_error_dialog	(const char *class,
 					 const char *format,
 					 ...) G_GNUC_PRINTF (2, 3);
@@ -100,10 +86,6 @@ char *		panel_gnome_kde_help_path (const char *docpath);
 
 gboolean	panel_is_url		(const char *url);
 
-/* Quote string so that it could be on a shell like
- * quoted line as an argument */
-char *		panel_quote_string	(const char *str);
-
 void		panel_push_window_busy	(GtkWidget *window);
 void		panel_pop_window_busy	(GtkWidget *window);
 
@@ -112,6 +94,25 @@ gboolean	panel_is_program_in_path (const char *program);
 char *		panel_pixmap_discovery	(const char *name,
 					 gboolean fallback);
 
+void		panel_stretch_events_to_toplevel (GtkWidget *widget,
+						 gboolean top,
+						 gboolean right,
+						 gboolean bottom,
+						 gboolean left);
+
+
+void		panel_signal_connect_while_alive (GObject     *object,
+						  const gchar *signal,
+						  GCallback    func,
+						  gpointer     func_data,
+						  GObject     *alive_object);
+void		panel_signal_connect_object_while_alive (GObject      *object,
+							 const gchar  *signal,
+							 GCallback     func,
+							 GObject      *alive_object);
+
+
+gboolean	panel_ensure_dir	(const char *dirname);
 
 /* GnomeVFS reading utils */
 
@@ -127,6 +128,10 @@ ReadBuf *	readbuf_open		(const char *uri);
 gboolean	readbuf_rewind		(ReadBuf *rb);
 void		readbuf_close		(ReadBuf *rb);
 
+
+/* FIXME : make battery stuff public as appropriate */
+
+
 G_END_DECLS
 
 #endif /* PANEL_UTIL_H */
