@@ -30,8 +30,7 @@ void send_state_change(void);
 
 GtkWidget * make_popup_panel_menu (PanelWidget *panel);
 
-PanelData * panel_data_by_id (int id);
-void panel_set_id (GtkWidget *widget, int id);
+PanelData * panel_data_by_id (const char *id);
 
 void status_unparent (GtkWidget *widget);
 
@@ -41,6 +40,14 @@ void status_unparent (GtkWidget *widget);
 	 (PANEL_WIDGET(GTK_WIDGET(appletw)->parent)->panel_parent)
 
 
+void panel_session_init_global_config (void);
+void panel_session_save_global_config (void);
+void panel_session_apply_global_config(void);
+
+void panel_session_init_panels (void);
+void panel_session_save_panel (PanelData *pd);
+void panel_session_remove_panel_from_config (PanelWidget *panel);
+
 G_END_DECLS
 
 #endif
