@@ -42,24 +42,15 @@ void panel_config_sync_schedule (void);
 
 void load_system_wide (void);
 
+gchar * session_get_current_profile (void);
+void session_set_current_profile (const gchar *profile_name);
+
 void session_load (void);
 
-void session_read_global_config (void);
-/* only to be used on beginning, we don't really want to write globals much */
-void session_write_global_config (void);
-
-
-void apply_global_config(void);
-
-/* just temporary so that we can bridge the gap between old and new config */
-void convert_old_config(void);
-
-/* prefix thingie */
-char * get_correct_prefix (char const **sep);
-void push_correct_global_prefix (void);
-
 void session_add_dead_launcher (const gchar *location);
 
+void panel_session_setup_config_sync (void);
+
 G_END_DECLS
 
 #endif
