@@ -8,7 +8,7 @@
 #include "config.h"
 
 #include "sliding-widget.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
 #include "foobar-widget.h"
 #include "panel-marshal.h"
 #include "panel-util.h"
@@ -278,7 +278,8 @@ sliding_pos_get_pos (BasePWidget *basep, int *x, int *y,
 }
 
 GtkWidget *
-sliding_widget_new (int screen,
+sliding_widget_new (gchar *panel_id,
+		    int screen,
 		    SlidingAnchor anchor,
 		    gint16 offset,
 		    BorderEdge edge,
@@ -290,19 +291,20 @@ sliding_widget_new (int screen,
 		    PanelBackType back_type,
 		    char *back_pixmap,
 		    gboolean fit_pixmap_bg,
-		    gboolean strech_pixmap_bg,
+		    gboolean stretch_pixmap_bg,
 		    gboolean rotate_pixmap_bg,
 		    GdkColor *back_color)
 {
-	SlidingWidget *sliding = gtk_type_new (SLIDING_TYPE_WIDGET);
-	SlidingPos *pos = gtk_type_new (SLIDING_TYPE_POS);
+	SlidingWidget *sliding = g_object_new (SLIDING_TYPE_WIDGET, NULL);
+	SlidingPos *pos = g_object_new (SLIDING_TYPE_POS, NULL);
 
 	pos->anchor = anchor;
 	pos->offset = offset;
 	
 	BASEP_WIDGET (sliding)->pos = BASEP_POS (pos);
 
-	border_widget_construct (BORDER_WIDGET (sliding),
+	border_widget_construct (panel_id,
+				 BORDER_WIDGET (sliding),
 				 screen,
 				 edge,
 				 TRUE,
@@ -314,7 +316,7 @@ sliding_widget_new (int screen,
 				 hidebutton_pixmaps_enabled,
 				 back_type,
 				 back_pixmap,
-				 fit_pixmap_bg, strech_pixmap_bg,
+				 fit_pixmap_bg, stretch_pixmap_bg,
 				 rotate_pixmap_bg,
 				 back_color);
 
@@ -335,7 +337,7 @@ sliding_widget_change_params (SlidingWidget *sliding,
 			      PanelBackType back_type,
 			      char *pixmap_name,
 			      gboolean fit_pixmap_bg,
-			      gboolean strech_pixmap_bg,
+			      gboolean stretch_pixmap_bg,
 			      gboolean rotate_pixmap_bg,
 			      GdkColor *back_color)
 {
@@ -367,7 +369,7 @@ sliding_widget_change_params (SlidingWidget *sliding,
 				     back_type,
 				     pixmap_name,
 				     fit_pixmap_bg,
-				     strech_pixmap_bg,
+				     stretch_pixmap_bg,
 				     rotate_pixmap_bg,
 				     back_color);
 }
@@ -395,7 +397,7 @@ sliding_widget_change_offset (SlidingWidget *sliding, gint16 offset)
 				      panel->back_type,
 				      panel->back_pixmap,
 				      panel->fit_pixmap_bg,
-				      panel->strech_pixmap_bg,
+				      panel->stretch_pixmap_bg,
 				      panel->rotate_pixmap_bg,
 				      &panel->back_color);
 }
@@ -423,7 +425,7 @@ sliding_widget_change_anchor (SlidingWidget *sliding, SlidingAnchor anchor)
 				      panel->back_type,
 				      panel->back_pixmap,
 				      panel->fit_pixmap_bg,
-				      panel->strech_pixmap_bg,
+				      panel->stretch_pixmap_bg,
 				      panel->rotate_pixmap_bg,
 				      &panel->back_color);
 }
@@ -450,7 +452,7 @@ sliding_widget_change_anchor_offset_edge (SlidingWidget *sliding,
 				      panel->back_type,
 				      panel->back_pixmap,
 				      panel->fit_pixmap_bg,
-				      panel->strech_pixmap_bg,
+				      panel->stretch_pixmap_bg,
 				      panel->rotate_pixmap_bg,
 				      &panel->back_color);
 }
