@@ -27,16 +27,15 @@
 #include "panel.h"
 #include "panel-util.h"
 #include "panel-widget.h"
+#include "panel-gconf.h"
+#include "session.h"
 #include "xstuff.h"
 
 /* from gtkhandlebox.c */
 #define DRAG_HANDLE_SIZE 10
 
 enum {
-	OK_BUTTON,
-	HELP_BUTTON,
-	RELOAD_BUTTON,
-	CANCEL_BUTTON
+	RELOAD_BUTTON
 };
 
 GList *check_swallows = NULL;
@@ -47,7 +46,7 @@ static void socket_realized (GtkWidget *w, gpointer data);
 static void
 socket_realized (GtkWidget *w, gpointer data)
 {
-	Swallow *swallow = gtk_object_get_user_data (GTK_OBJECT (w));
+	Swallow *swallow = data;
 
 	g_return_if_fail (swallow->title != NULL);
 	
@@ -78,7 +77,10 @@ swallow_launch (Swallow *swallow)
 			char *curdir = g_get_current_dir ();
 			chdir (g_get_home_dir ());
 
-			gnome_desktop_item_launch (item, 0, NULL, NULL);
+			gnome_desktop_item_launch (item,
+						   NULL /* file_list */,
+						   0 /* flags */,
+						   NULL /* error */);
 			/* FIXME: handle_errors */
 			gnome_desktop_item_unref (item);
 
@@ -108,7 +110,7 @@ before_remove (Swallow *swallow)
 
 	gtk_dialog_add_button (GTK_DIALOG (dlg),
 			       GTK_STOCK_CANCEL,
-			       CANCEL_BUTTON);
+			       GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (dlg),
 			       _("Reload"),
 			       RELOAD_BUTTON);
@@ -121,17 +123,15 @@ before_remove (Swallow *swallow)
 
 		/* make the socket again */
 		swallow->socket = gtk_socket_new ();
-		gtk_object_set_user_data (GTK_OBJECT (swallow->socket),
-					  swallow);
 
-		if (swallow->width != 0 || swallow->height != 0)
-			gtk_widget_set_usize(swallow->socket,
-					     swallow->width, swallow->height);
+		if (swallow->width > 0 || swallow->height > 0)
+			gtk_widget_set_size_request (swallow->socket,
+						     swallow->width, swallow->height);
 
 		g_signal_connect_after (G_OBJECT (swallow->socket),
 					"realize",
 					G_CALLBACK (socket_realized),
-					NULL);
+					swallow);
 		g_signal_connect (G_OBJECT (swallow->socket), "destroy",
 				  G_CALLBACK (socket_destroyed),
 				  swallow);
@@ -197,20 +197,20 @@ socket_destroyed (GtkWidget *w, gpointer data)
 static void
 really_add_swallow (GtkWidget *d, int response, gpointer data)
 {
-	GtkWidget *title_e = gtk_object_get_data(GTK_OBJECT(d), "title_e");
-	GtkWidget *exec_e = gtk_object_get_data(GTK_OBJECT(d), "exec_e");
-	GtkWidget *width_s = gtk_object_get_data(GTK_OBJECT(d), "width_s");
-	GtkWidget *height_s = gtk_object_get_data(GTK_OBJECT(d), "height_s");
-	PanelWidget *panel = gtk_object_get_data(GTK_OBJECT(d), "panel");
-	int pos = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(d), "pos"));
+	GtkWidget *title_e = g_object_get_data (G_OBJECT (d), "title_e");
+	GtkWidget *exec_e = g_object_get_data (G_OBJECT (d), "exec_e");
+	GtkWidget *width_s = g_object_get_data (G_OBJECT (d), "width_s");
+	GtkWidget *height_s = g_object_get_data (G_OBJECT (d), "height_s");
+	PanelWidget *panel = g_object_get_data (G_OBJECT (d), "panel");
+	int pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (d), "pos"));
 	gboolean exactpos =
-		GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(d), "exactpos"));
+		GPOINTER_TO_INT (g_object_get_data (G_OBJECT (d), "exactpos"));
 
 	switch (response) {
-	case CANCEL_BUTTON:
+	case GTK_RESPONSE_CANCEL:
 		gtk_widget_destroy (d);
 		return;
-	case HELP_BUTTON:
+	case GTK_RESPONSE_HELP:
 		panel_show_help ("specialobjects", "SWALLOWEDAPPS");
 		return;
 	default:
@@ -223,7 +223,7 @@ really_add_swallow (GtkWidget *d, int response, gpointer data)
 						GTK_SPIN_BUTTON(width_s)),
 			    gtk_spin_button_get_value_as_int(
 						GTK_SPIN_BUTTON(height_s)),
-			    panel, pos, exactpos);
+			    panel, pos, exactpos, NULL);
 	gtk_widget_destroy(d);
 }
 
@@ -253,21 +253,19 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	d = gtk_dialog_new_with_buttons (_("Create swallow applet"),
 					 NULL /* parent */,
 					 0 /* flags */,
-					 GTK_STOCK_CANCEL,
-					 CANCEL_BUTTON,
-					 GTK_STOCK_OK,
-					 OK_BUTTON,
 					 GTK_STOCK_HELP,
-					 HELP_BUTTON,
+					 GTK_RESPONSE_HELP,
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_OK,
+					 GTK_RESPONSE_OK,
 					 NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (d),
 				"create_swallow", "Panel");
-	/*gtk_window_set_position(GTK_WINDOW(d), GTK_WIN_POS_CENTER);*/
-	gtk_window_set_policy(GTK_WINDOW(d), FALSE, FALSE, TRUE);
 
-	gtk_object_set_data(GTK_OBJECT(d),"panel",panel);
-	gtk_object_set_data(GTK_OBJECT(d),"pos",GINT_TO_POINTER(pos));
-	gtk_object_set_data(GTK_OBJECT(d),"exactpos",GINT_TO_POINTER(exactpos));
+	g_object_set_data (G_OBJECT (d), "panel", panel);
+	g_object_set_data (G_OBJECT (d), "pos", GINT_TO_POINTER (pos));
+	g_object_set_data (G_OBJECT (d), "exactpos", GINT_TO_POINTER (exactpos));
 
 	box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),box, TRUE,TRUE,0);
@@ -316,18 +314,17 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	
 	g_signal_connect (G_OBJECT (d), "response",
 			  G_CALLBACK (really_add_swallow), NULL);
-	gtk_object_set_data(GTK_OBJECT(d),"title_e",title_e);
-	gtk_object_set_data(GTK_OBJECT(d),"exec_e",exec_e);
-	gtk_object_set_data(GTK_OBJECT(d),"width_s",width_s);
-	gtk_object_set_data(GTK_OBJECT(d),"height_s",height_s);
+	g_object_set_data (G_OBJECT (d), "title_e", title_e);
+	g_object_set_data (G_OBJECT (d), "exec_e", exec_e);
+	g_object_set_data (G_OBJECT (d), "width_s", width_s);
+	g_object_set_data (G_OBJECT (d), "height_s", height_s);
 
 
-	gtk_dialog_set_default_response (GTK_DIALOG (d), OK_BUTTON);
+	gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
 
 	gtk_widget_grab_focus(title_e);
 
 	gtk_widget_show_all (d);
-	panel_set_dialog_layer (d);
 }
 
 static int
@@ -349,6 +346,11 @@ static Swallow *
 create_swallow_applet(const char *title, const char *path, int width, int height, SwallowOrient orient)
 {
 	Swallow *swallow;
+
+	if (width == 0)
+		width = -1;
+	if (height == 0)
+		height = -1;
 	
 	swallow = g_new (Swallow, 1);
 	swallow->ref_count = 1;
@@ -359,10 +361,9 @@ create_swallow_applet(const char *title, const char *path, int width, int height
 	gtk_widget_show (swallow->ebox);
 
 	swallow->socket = gtk_socket_new ();
-	if (width != 0 || height != 0)
-		gtk_widget_set_usize(swallow->socket, width, height);
+	gtk_widget_set_size_request (swallow->socket, width, height);
 	g_signal_connect_after (G_OBJECT (swallow->socket), "realize",
-				G_CALLBACK (socket_realized), NULL);
+				G_CALLBACK (socket_realized), swallow);
 	g_signal_connect (G_OBJECT (swallow->socket), "destroy",
 			  G_CALLBACK (socket_destroyed), swallow);
 	
@@ -384,7 +385,6 @@ create_swallow_applet(const char *title, const char *path, int width, int height
 			   swallow->socket);
 
 	gtk_widget_show (swallow->socket);
-	gtk_object_set_user_data (GTK_OBJECT (swallow->socket), swallow);
 
 	swallow->title = g_strdup (title);
 	swallow->path = path ? g_strdup (path) : NULL;
@@ -403,30 +403,36 @@ set_swallow_applet_orient(Swallow *swallow, SwallowOrient orient)
 	if (GTK_HANDLE_BOX(swallow->handle_box)->child_detached) {
 		if (orient == SWALLOW_VERTICAL) {
 			GTK_HANDLE_BOX(swallow->handle_box)->handle_position = GTK_POS_TOP;
-			gtk_widget_set_usize (swallow->handle_box, swallow->width,
-					      DRAG_HANDLE_SIZE);
+			gtk_widget_set_size_request (swallow->handle_box, swallow->width,
+						     DRAG_HANDLE_SIZE);
 		} else {
 			GTK_HANDLE_BOX(swallow->handle_box)->handle_position = GTK_POS_LEFT;
-			gtk_widget_set_usize (swallow->handle_box, DRAG_HANDLE_SIZE,
-					      swallow->height);
+			gtk_widget_set_size_request (swallow->handle_box, DRAG_HANDLE_SIZE,
+						     swallow->height);
 		}
 	} else {
 		if (orient == SWALLOW_VERTICAL) {
 			GTK_HANDLE_BOX(swallow->handle_box)->handle_position = GTK_POS_TOP;
-			gtk_widget_set_usize (swallow->handle_box, swallow->width,
-					      swallow->height + DRAG_HANDLE_SIZE);
+			gtk_widget_set_size_request (swallow->handle_box, swallow->width,
+						     swallow->height + DRAG_HANDLE_SIZE);
 		} else {
 			GTK_HANDLE_BOX(swallow->handle_box)->handle_position = GTK_POS_LEFT;
-			gtk_widget_set_usize (swallow->handle_box,
-					      swallow->width + DRAG_HANDLE_SIZE,
-					      swallow->height);
+			gtk_widget_set_size_request (swallow->handle_box,
+						     swallow->width + DRAG_HANDLE_SIZE,
+						     swallow->height);
 		}
 	}
 }
 
 void
-load_swallow_applet (const char *path, const char *params, int width, int height,
-		     PanelWidget *panel, int pos, gboolean exactpos)
+load_swallow_applet (const char  *path,
+		     const char  *params,
+		     int          width,
+		     int          height,
+		     PanelWidget *panel,
+		     int          pos,
+		     gboolean     exactpos,
+		     const char  *gconf_key)
 {
 	Swallow    *swallow;
 	AppletInfo *info;
@@ -438,7 +444,8 @@ load_swallow_applet (const char *path, const char *params, int width, int height
 
 	info = panel_applet_register (swallow->ebox, ref_swallow (swallow),
 				      (GDestroyNotify) unref_swallow,
-				      panel, pos, exactpos, APPLET_SWALLOW);
+				      panel, pos, exactpos, APPLET_SWALLOW,
+				      gconf_key);
 	if (!info)
 		return;
 
@@ -448,3 +455,83 @@ load_swallow_applet (const char *path, const char *params, int width, int height
 
 	swallow_launch (swallow);
 }
+
+void
+swallow_save_to_gconf (Swallow    *swallow,
+		       const char *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+
+	g_return_if_fail (swallow);
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "exec-path");
+	gconf_client_set_string (client, temp_key, swallow->path, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "parameters");
+	gconf_client_set_string (client, temp_key, swallow->title, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "width");
+	gconf_client_set_int (client, temp_key, swallow->width, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "height");
+	gconf_client_set_int (client, temp_key, swallow->height, NULL);
+	g_free (temp_key);
+	
+	g_free (profile);
+}
+
+void
+swallow_load_from_gconf (PanelWidget *panel_widget,
+			 gint         position,
+			 const char  *gconf_key,
+			 gboolean     use_default)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+	char        *path;
+	char        *params;
+	int          width;
+	int          height;
+
+	g_return_if_fail (panel_widget);
+	g_return_if_fail (gconf_key);
+
+	client  = panel_gconf_get_client ();
+	if (use_default)
+		profile = g_strdup ("medium");
+	else
+		profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "exec-path", use_default);
+	path = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "parameters", use_default);
+	params = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "width", use_default);
+	width = gconf_client_get_int (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "height", use_default);
+	height = gconf_client_get_int (client, temp_key, NULL);
+	g_free (temp_key);
+
+	g_free (profile);
+
+	load_swallow_applet (path, params, width, height, panel_widget, position, TRUE, gconf_key);
+
+	g_free (path);
+	g_free (params);
+	g_free (profile);
+}
