@@ -47,7 +47,8 @@ void xstuff_unsetup_desktop_area	(void);
 void xstuff_set_pos_size		(GdkWindow *window,
 					 int x, int y,
 					 int w, int h);
-void xstuff_set_wmspec_dock_hints       (GdkWindow *window);
+void xstuff_set_wmspec_dock_hints       (GdkWindow *window,
+					 gboolean autohide);
 void xstuff_set_wmspec_strut		(GdkWindow *window,
 					 int left,
 					 int right,
